@@ -413,6 +413,16 @@ pub struct PipCompileArgs {
     #[arg(long, overrides_with("no_strip_extras"), hide = true)]
     pub strip_extras: bool,
 
+    /// Include environment markers in the output file.
+    ///
+    /// By default, `uv` strips environment markers, as the resolution generated by `compile` is
+    /// only guaranteed to be correct for the target environment.
+    #[arg(long, overrides_with("strip_markers"))]
+    pub no_strip_markers: bool,
+
+    #[arg(long, overrides_with("no_strip_markers"), hide = true)]
+    pub strip_markers: bool,
+
     /// Exclude comment annotations indicating the source of each package.
     #[arg(long, overrides_with("annotate"))]
     pub no_annotate: bool,
@@ -558,6 +568,15 @@ pub struct PipCompileArgs {
     #[arg(long)]
     pub python_platform: Option<TargetTriple>,
 
+    /// Perform a universal resolution, attempting to generate a single `requirements.txt` output
+    /// file that is compatible with all operating systems, architectures and supported Python
+    /// versions.
+    #[arg(long, overrides_with("no_universal"))]
+    pub universal: bool,
+
+    #[arg(long, overrides_with("universal"), hide = true)]
+    pub no_universal: bool,
+
     /// Specify a package to omit from the output resolution. Its dependencies will still be
     /// included in the resolution. Equivalent to pip-compile's `--unsafe-package` option.
     #[arg(long, alias = "unsafe-package")]
@@ -1385,6 +1404,14 @@ pub struct PipShowArgs {
 #[derive(Args)]
 #[allow(clippy::struct_excessive_bools)]
 pub struct PipTreeArgs {
+    /// Do not de-duplicate repeated dependencies.
+    /// Usually, when a package has already displayed its dependencies,
+    /// further occurrences will not re-display its dependencies,
+    /// and will include a (*) to indicate it has already been shown.
+    /// This flag will cause those duplicates to be repeated.
+    #[arg(long)]
+    pub no_dedupe: bool,
+
     /// Validate the virtual environment, to detect packages with missing dependencies or other
     /// issues.
     #[arg(long, overrides_with("no_strict"))]
@@ -1612,6 +1639,10 @@ pub struct RunArgs {
     #[command(flatten)]
     pub refresh: RefreshArgs,
 
+    /// Run the command in a specific package in the workspace.
+    #[arg(long, conflicts_with = "isolated")]
+    pub package: Option<PackageName>,
+
     /// The Python interpreter to use to build the run environment.
     ///
     /// By default, `uv` uses the virtual environment in the current working directory or any parent
@@ -1625,10 +1656,6 @@ pub struct RunArgs {
     /// - `/home/ferris/.local/bin/python3.10` uses the exact Python at the given path.
     #[arg(long, short, env = "UV_PYTHON", verbatim_doc_comment)]
     pub python: Option<String>,
-
-    /// Run the command in a different package in the workspace.
-    #[arg(long, conflicts_with = "isolated")]
-    pub package: Option<PackageName>,
 }
 
 #[derive(Args)]
@@ -1722,10 +1749,6 @@ pub struct AddArgs {
     #[arg(long)]
     pub dev: bool,
 
-    /// Add the requirements as workspace dependencies.
-    #[arg(long)]
-    pub workspace: bool,
-
     /// Add the requirements as editables.
     #[arg(long, default_missing_value = "true", num_args(0..=1))]
     pub editable: Option<bool>,
@@ -1734,7 +1757,7 @@ pub struct AddArgs {
     ///
     /// Without this flag uv will try to use `tool.uv.sources` for any sources.
     #[arg(long)]
-    pub raw: bool,
+    pub raw_sources: bool,
 
     /// Specific commit to use when adding from Git.
     #[arg(long)]
@@ -1748,6 +1771,10 @@ pub struct AddArgs {
     #[arg(long)]
     pub branch: Option<String>,
 
+    /// Extras to activate for the dependency; may be provided more than once.
+    #[arg(long)]
+    pub extra: Option<Vec<ExtraName>>,
+
     #[command(flatten)]
     pub installer: ResolverInstallerArgs,
 
@@ -1757,6 +1784,10 @@ pub struct AddArgs {
     #[command(flatten)]
     pub refresh: RefreshArgs,
 
+    /// Add the dependency to a specific package in the workspace.
+    #[arg(long, conflicts_with = "isolated")]
+    pub package: Option<PackageName>,
+
     /// The Python interpreter into which packages should be installed.
     ///
     /// By default, `uv` installs into the virtual environment in the current working directory or
@@ -1784,6 +1815,10 @@ pub struct RemoveArgs {
     #[arg(long)]
     pub dev: bool,
 
+    /// Remove the dependency from a specific package in the workspace.
+    #[arg(long, conflicts_with = "isolated")]
+    pub package: Option<PackageName>,
+
     /// The Python interpreter into which packages should be installed.
     ///
     /// By default, `uv` installs into the virtual environment in the current working directory or
@@ -1811,6 +1846,8 @@ pub struct ToolNamespace {
 pub enum ToolCommand {
     /// Run a tool
     Run(ToolRunArgs),
+    /// Install a tool
+    Install(ToolInstallArgs),
 }
 
 #[derive(Args)]
@@ -1854,6 +1891,52 @@ pub struct ToolRunArgs {
     pub python: Option<String>,
 }
 
+#[derive(Args)]
+#[allow(clippy::struct_excessive_bools)]
+pub struct ToolInstallArgs {
+    /// The command to install.
+    pub name: String,
+
+    /// Use the given package to provide the command.
+    ///
+    /// By default, the package name is assumed to match the command name.
+    #[arg(long)]
+    pub from: Option<String>,
+
+    /// Include the following extra requirements.
+    #[arg(long)]
+    pub with: Vec<String>,
+
+    #[command(flatten)]
+    pub installer: ResolverInstallerArgs,
+
+    #[command(flatten)]
+    pub build: BuildArgs,
+
+    #[command(flatten)]
+    pub refresh: RefreshArgs,
+
+    /// Force installation of the tool.
+    ///
+    /// Will replace any existing entry points with the same name in the executable directory.
+    #[arg(long)]
+    pub force: bool,
+
+    /// The Python interpreter to use to build the tool environment.
+    ///
+    /// By default, uv will search for a Python executable in the `PATH`. uv ignores virtual
+    /// environments while looking for interpreter for tools. The `--python` option allows
+    /// you to specify a different interpreter.
+    ///
+    /// Supported formats:
+    /// - `3.10` looks for an installed Python 3.10 using `py --list-paths` on Windows, or
+    ///   `python3.10` on Linux and macOS.
+    /// - `python3.10` or `python.exe` looks for a binary with the given name in `PATH`.
+    /// - `/home/ferris/.local/bin/python3.10` uses the exact Python at the given path.
+    #[arg(long, short, env = "UV_PYTHON", verbatim_doc_comment)]
+    pub python: Option<String>,
+}
+
 #[derive(Args)]
 #[allow(clippy::struct_excessive_bools)]
 pub struct ToolchainNamespace {
