@@ -220,7 +220,7 @@ impl RegistryClient {
 
         let mut results = Vec::new();
         for index in it {
-            match self.simple_single_index(package_name, index).await? {
+            match self.simple_single_index(package_name, index).await {
                 Ok(metadata) => {
                     results.push((index.clone(), metadata));
 
@@ -229,8 +229,11 @@ impl RegistryClient {
                         break;
                     }
                 }
-                Err(CachedClientError::Client(err)) => match err.into_kind() {
+                Err(err) => match err.into_kind() {
+                    // The package is unavailable due to a lack of connectivity.
                     ErrorKind::Offline(_) => continue,
+
+                    // The package could not be found in the remote index.
                     ErrorKind::ReqwestError(err) => {
                         if err.status() == Some(StatusCode::NOT_FOUND)
                             || err.status() == Some(StatusCode::UNAUTHORIZED)
@@ -240,9 +243,12 @@ impl RegistryClient {
                         }
                         return Err(ErrorKind::from(err).into());
                     }
+
+                    // The package could not be found in the local index.
+                    ErrorKind::FileNotFound(_) => continue,
+
                     other => return Err(other.into()),
                 },
-                Err(CachedClientError::Callback(err)) => return Err(err),
             };
         }
 
@@ -266,11 +272,11 @@ impl RegistryClient {
         &self,
         package_name: &PackageName,
         index: &IndexUrl,
-    ) -> Result<Result<OwnedArchive<SimpleMetadata>, CachedClientError<Error>>, Error> {
+    ) -> Result<OwnedArchive<SimpleMetadata>, Error> {
         // Format the URL for PyPI.
         let mut url: Url = index.clone().into();
         url.path_segments_mut()
-            .unwrap()
+            .map_err(|()| ErrorKind::CannotBeABase(index.clone().into()))?
             .pop_if_empty()
             .push(package_name.as_ref())
             // The URL *must* end in a trailing slash for proper relative path behavior
@@ -298,7 +304,7 @@ impl RegistryClient {
         };
 
         if matches!(index, IndexUrl::Path(_)) {
-            self.fetch_local_index(package_name, &url).await.map(Ok)
+            self.fetch_local_index(package_name, &url).await
         } else {
             self.fetch_remote_index(package_name, &url, &cache_entry, cache_control)
                 .await
@@ -312,7 +318,7 @@ impl RegistryClient {
         url: &Url,
         cache_entry: &CacheEntry,
         cache_control: CacheControl,
-    ) -> Result<Result<OwnedArchive<SimpleMetadata>, CachedClientError<Error>>, Error> {
+    ) -> Result<OwnedArchive<SimpleMetadata>, Error> {
         let simple_request = self
             .uncached_client()
             .get(url.clone())
@@ -359,16 +365,18 @@ impl RegistryClient {
             .boxed_local()
             .instrument(info_span!("parse_simple_api", package = %package_name))
         };
-        let result = self
-            .cached_client()
+        self.cached_client()
             .get_cacheable(
                 simple_request,
                 cache_entry,
                 cache_control,
                 parse_simple_response,
             )
-            .await;
-        Ok(result)
+            .await
+            .map_err(|err| match err {
+                CachedClientError::Client(err) => err,
+                CachedClientError::Callback(err) => err,
+            })
     }
 
     /// Fetch the [`SimpleMetadata`] from a local file, using a PEP 503-compatible directory
@@ -382,9 +390,17 @@ impl RegistryClient {
             .to_file_path()
             .map_err(|()| ErrorKind::NonFileUrl(url.clone()))?
             .join("index.html");
-        let text = fs_err::tokio::read_to_string(&path)
-            .await
-            .map_err(ErrorKind::from)?;
+        let text = match fs_err::tokio::read_to_string(&path).await {
+            Ok(text) => text,
+            Err(err) if err.kind() == std::io::ErrorKind::NotFound => {
+                return Err(Error::from(ErrorKind::FileNotFound(
+                    package_name.to_string(),
+                )));
+            }
+            Err(err) => {
+                return Err(Error::from(ErrorKind::Io(err)));
+            }
+        };
         let metadata = SimpleMetadata::from_html(&text, package_name, url)?;
         OwnedArchive::from_unarchived(&metadata)
     }
