@@ -1,7 +1,9 @@
 pub use authentication::*;
 pub use build_options::*;
+pub use concurrency::*;
 pub use config_settings::*;
 pub use constraints::*;
+pub use extras::*;
 pub use name_specifiers::*;
 pub use overrides::*;
 pub use package_options::*;
@@ -10,8 +12,10 @@ pub use target_triple::*;
 
 mod authentication;
 mod build_options;
+mod concurrency;
 mod config_settings;
 mod constraints;
+mod extras;
 mod name_specifiers;
 mod overrides;
 mod package_options;
