@@ -1,12 +1,12 @@
 pub use compile::{compile_tree, CompileError};
-pub use downloader::{Downloader, Reporter as DownloadReporter};
 pub use installer::{Installer, Reporter as InstallReporter};
 pub use plan::{Plan, Planner};
+pub use preparer::{Preparer, Reporter as PrepareReporter};
 pub use site_packages::{SatisfiesResult, SitePackages, SitePackagesDiagnostic};
 pub use uninstall::{uninstall, UninstallError};
 
 mod compile;
-mod downloader;
+mod preparer;
 
 mod installer;
 mod plan;
