@@ -263,6 +263,7 @@ async fn run() -> Result<ExitStatus> {
                 args.settings.generate_hashes,
                 args.settings.no_emit_package,
                 args.settings.no_strip_extras,
+                args.settings.no_strip_markers,
                 !args.settings.no_annotate,
                 !args.settings.no_header,
                 args.settings.custom_compile_command,
@@ -281,6 +282,7 @@ async fn run() -> Result<ExitStatus> {
                 args.settings.build_options,
                 args.settings.python_version,
                 args.settings.python_platform,
+                args.settings.universal,
                 args.settings.exclude_newer,
                 args.settings.annotation_style,
                 args.settings.link_mode,
@@ -546,6 +548,7 @@ async fn run() -> Result<ExitStatus> {
             let cache = cache.init()?;
 
             commands::pip_tree(
+                args.no_dedupe,
                 args.shared.strict,
                 args.shared.python.as_deref(),
                 args.shared.system,
@@ -716,13 +719,14 @@ async fn run() -> Result<ExitStatus> {
 
             commands::add(
                 args.requirements,
-                args.workspace,
                 args.dev,
                 args.editable,
-                args.raw,
+                args.raw_sources,
                 args.rev,
                 args.tag,
                 args.branch,
+                args.extras,
+                args.package,
                 args.python,
                 args.settings,
                 globals.toolchain_preference,
@@ -746,6 +750,7 @@ async fn run() -> Result<ExitStatus> {
             commands::remove(
                 args.requirements,
                 args.dev,
+                args.package,
                 args.python,
                 globals.toolchain_preference,
                 globals.preview,
@@ -796,6 +801,33 @@ async fn run() -> Result<ExitStatus> {
             )
             .await
         }
+        Commands::Tool(ToolNamespace {
+            command: ToolCommand::Install(args),
+        }) => {
+            // Resolve the settings from the command-line arguments and workspace configuration.
+            let args = settings::ToolInstallSettings::resolve(args, filesystem);
+            show_settings!(args);
+
+            // Initialize the cache.
+            let cache = cache.init()?.with_refresh(args.refresh);
+
+            commands::tool_install(
+                args.name,
+                args.python,
+                args.from,
+                args.with,
+                args.force,
+                args.settings,
+                globals.preview,
+                globals.toolchain_preference,
+                globals.connectivity,
+                Concurrency::default(),
+                globals.native_tls,
+                &cache,
+                printer,
+            )
+            .await
+        }
         Commands::Toolchain(ToolchainNamespace {
             command: ToolchainCommand::List(args),
         }) => {
